'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const $RefParser = require('json-schema-ref-parser');
const isNil = require('inspected/schema/is-nil').default;
const isString = require('inspected/schema/is-string').default;
const isObject = require('inspected/schema/is-object').default;
const merge = require('deepmerge');
const fs = require('fs');

const fromFile = filePath => {
  return new Promise((resolve, reject) => {
    fs.readFile(filePath, 'utf8', (err, data) => {
      if (err) {
        reject(err);
        return;
      }

      const spec = JSON.parse(data);
      resolve(spec);
    });
  });
};

const dereference = (() => {
  var _ref = _asyncToGenerator(function* (content, basePath, parser, resolver) {
    const options = {
      parse: { custom: parser },
      resolve: { custom: resolver }
    };

    if (basePath) {
      return yield $RefParser.dereference(basePath, content, options);
    }

    return yield $RefParser.dereference(content, options);
  });

  return function dereference(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
})();

const parse = (() => {
  var _ref2 = _asyncToGenerator(function* (content, options) {
    if (isNil(content)) {
      throw new Error('No content specified.');
    }

    if (!isString(content) && !isObject(content)) {
      throw new Error('Content must be a string or spec object.');
    }

    // TODO: validate options against schema
    if (!isNil(options) && !isObject(options)) {
      throw new Error('Options should be an object.');
    }

    try {
      const defaultOptions = {
        parser: {
          canParse: function (file) {
            return false;
          },
          parse: (() => {
            var _ref3 = _asyncToGenerator(function* (file) {});

            return function parse(_x7) {
              return _ref3.apply(this, arguments);
            };
          })()
        },
        resolver: {
          canResolve: function (info) {
            return false;
          },
          resolve: (() => {
            var _ref4 = _asyncToGenerator(function* (info) {});

            return function resolve(_x8) {
              return _ref4.apply(this, arguments);
            };
          })()
        }
      };

      const compiledOptions = options ? merge(defaultOptions, options) : defaultOptions;

      const parser = {
        order: 1,
        canParse: function (info) {
          return compiledOptions.parser.canParse(info);
        },
        parse: (() => {
          var _ref5 = _asyncToGenerator(function* (info) {
            return compiledOptions.parser.parse(info);
          });

          return function parse(_x9) {
            return _ref5.apply(this, arguments);
          };
        })()
      };

      const resolver = {
        order: 1,
        canRead: function (info) {
          return compiledOptions.resolver.canResolve({
            path: info.url,
            extension: info.extension
          });
        },
        read: (() => {
          var _ref6 = _asyncToGenerator(function* (info) {
            return yield compiledOptions.resolver.resolve({
              path: info.url,
              extension: info.extension
            });
          });

          return function read(_x10) {
            return _ref6.apply(this, arguments);
          };
        })()
      };

      const spec = yield dereference(content, compiledOptions.basePath, parser, resolver);

      return spec;
    } catch (error) {
      throw new Error(`There was an error parsing the specified spec:\n${error}`);
    }
  });

  return function parse(_x5, _x6) {
    return _ref2.apply(this, arguments);
  };
})();

module.exports = parse;